"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.help = exports.run = void 0;
const fs_1 = require("fs");
const json5_1 = require("json5");
const path_1 = require("path");
const presets_1 = require("../helpers/presets");
const run = (args) => __awaiter(void 0, void 0, void 0, function* () {
    const file = args._[0] || 'jest.config.js';
    const filePath = path_1.join(process.cwd(), file);
    const name = path_1.basename(file);
    const isPackage = name === 'package.json';
    const exists = fs_1.existsSync(filePath);
    const pkgFile = isPackage ? filePath : path_1.join(process.cwd(), 'package.json');
    const hasPackage = isPackage || fs_1.existsSync(pkgFile);
    const { jestPreset = true, tsconfig: askedTsconfig, force, jsdom } = args;
    const tsconfig = askedTsconfig === 'tsconfig.json' ? undefined : askedTsconfig;
    const pkgJson = hasPackage ? JSON.parse(fs_1.readFileSync(pkgFile, 'utf8')) : {};
    let { js: jsFilesProcessor, babel: shouldPostProcessWithBabel } = args;
    if (jsFilesProcessor == null) {
        jsFilesProcessor = shouldPostProcessWithBabel ? 'babel' : undefined;
    }
    else if (shouldPostProcessWithBabel == null) {
        shouldPostProcessWithBabel = jsFilesProcessor === 'babel';
    }
    let preset;
    if (jsFilesProcessor === 'babel') {
        preset = presets_1.jsWIthBabel;
    }
    else if (jsFilesProcessor === 'ts') {
        preset = presets_1.jsWithTs;
    }
    else {
        preset = presets_1.defaults;
    }
    if (isPackage && !exists) {
        throw new Error(`File ${file} does not exists.`);
    }
    else if (!isPackage && exists && !force) {
        throw new Error(`Configuration file ${file} already exists.`);
    }
    if (!isPackage && !name.endsWith('.js')) {
        throw new TypeError(`Configuration file ${file} must be a .js file or the package.json.`);
    }
    if (hasPackage && pkgJson.jest) {
        if (force && !isPackage) {
            delete pkgJson.jest;
            fs_1.writeFileSync(pkgFile, JSON.stringify(pkgJson, undefined, '  '));
        }
        else if (!force) {
            throw new Error(`A Jest configuration is already set in ${pkgFile}.`);
        }
    }
    let body;
    if (isPackage) {
        const base = jestPreset ? { preset: preset.name } : Object.assign({}, preset.value);
        if (!jsdom)
            base.testEnvironment = 'node';
        if (tsconfig || shouldPostProcessWithBabel) {
            const tsJestConf = {};
            base.globals = { 'ts-jest': tsJestConf };
            if (tsconfig)
                tsJestConf.tsconfig = tsconfig;
            if (shouldPostProcessWithBabel)
                tsJestConf.babelConfig = true;
        }
        body = JSON.stringify(Object.assign(Object.assign({}, pkgJson), { jest: base }), undefined, '  ');
    }
    else {
        const content = [];
        if (!jestPreset) {
            content.push(`${preset.jsImport('tsjPreset')};`, '');
        }
        content.push('module.exports = {');
        if (jestPreset) {
            content.push(`  preset: '${preset.name}',`);
        }
        else {
            content.push('  ...tsjPreset,');
        }
        if (!jsdom)
            content.push("  testEnvironment: 'node',");
        if (tsconfig || shouldPostProcessWithBabel) {
            content.push('  globals: {');
            content.push("    'ts-jest': {");
            if (tsconfig)
                content.push(`      tsconfig: ${json5_1.stringify(tsconfig)},`);
            if (shouldPostProcessWithBabel)
                content.push('      babelConfig: true,');
            content.push('    },');
            content.push('  },');
        }
        content.push('};');
        body = content.join('\n');
    }
    fs_1.writeFileSync(filePath, body);
    process.stderr.write(`
Jest configuration written to "${filePath}".
`);
});
exports.run = run;
const help = () => __awaiter(void 0, void 0, void 0, function* () {
    process.stdout.write(`
Usage:
  ts-jest config:init [options] [<config-file>]

Arguments:
  <config-file>         Can be a js or json Jest config file. If it is a
                        package.json file, the configuration will be read from
                        the "jest" property.
                        Default: jest.config.js

Options:
  --force               Discard any existing Jest config
  --js ts|babel         Process .js files with ts-jest if 'ts' or with
                        babel-jest if 'babel'
  --no-jest-preset      Disable the use of Jest presets
  --tsconfig <file>     Path to the tsconfig.json file
  --babel               Pipe babel-jest after ts-jest
  --jsdom               Use jsdom as test environment instead of node
`);
});
exports.help = help;
