"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.help = exports.run = void 0;
const bs_logger_1 = require("bs-logger");
const stableStringify = require("fast-json-stable-stringify");
const fs_1 = require("fs");
const json5_1 = require("json5");
const path_1 = require("path");
const backports_1 = require("../../utils/backports");
const presets_1 = require("../helpers/presets");
const run = (args) => __awaiter(void 0, void 0, void 0, function* () {
    const nullLogger = bs_logger_1.createLogger({ targets: [] });
    const file = args._[0];
    const filePath = path_1.resolve(process.cwd(), file);
    const footNotes = [];
    if (!fs_1.existsSync(filePath)) {
        throw new Error(`Configuration file ${file} does not exists.`);
    }
    const name = path_1.basename(file);
    const isPackage = name === 'package.json';
    if (!/\.(js|json)$/.test(name)) {
        throw new TypeError(`Configuration file ${file} must be a JavaScript or JSON file.`);
    }
    let actualConfig = require(filePath);
    if (isPackage) {
        actualConfig = actualConfig.jest;
    }
    if (!actualConfig)
        actualConfig = {};
    const migratedConfig = backports_1.backportJestConfig(nullLogger, actualConfig);
    let presetName;
    let preset;
    if (!migratedConfig.preset && args.jestPreset) {
        if (args.js) {
            presetName = args.js === 'babel' ? "ts-jest/presets/js-with-babel" : "ts-jest/presets/js-with-ts";
        }
        else {
            const jsTransformers = Object.keys(migratedConfig.transform || {}).reduce((list, pattern) => {
                if (RegExp(pattern.replace(/^<rootDir>\/?/, '/dummy-project/')).test('/dummy-project/src/foo.js')) {
                    let transformer = migratedConfig.transform[pattern];
                    if (/\bbabel-jest\b/.test(transformer))
                        transformer = 'babel-jest';
                    else if (/\ts-jest\b/.test(transformer))
                        transformer = 'ts-jest';
                    return [...list, transformer];
                }
                return list;
            }, []);
            const jsWithTs = jsTransformers.includes('ts-jest');
            const jsWithBabel = jsTransformers.includes('babel-jest');
            if (jsWithBabel && !jsWithTs) {
                presetName = "ts-jest/presets/js-with-babel";
            }
            else if (jsWithTs && !jsWithBabel) {
                presetName = "ts-jest/presets/js-with-ts";
            }
            else {
                presetName = "ts-jest/presets/default";
            }
        }
        presetName = presetName || "ts-jest/presets/default";
        preset = presets_1.allPresets[presetName];
        footNotes.push(`Detected preset '${preset.label}' as the best matching preset for your configuration.
Visit https://kulshekhar.github.io/ts-jest/user/config/#jest-preset for more information about presets.
`);
    }
    else if (migratedConfig.preset && migratedConfig.preset.startsWith('ts-jest')) {
        if (args.jestPreset === false) {
            delete migratedConfig.preset;
        }
        else {
            preset = presets_1.allPresets[migratedConfig.preset] || presets_1.defaults;
        }
    }
    if (preset)
        migratedConfig.preset = preset.name;
    if (migratedConfig.moduleFileExtensions && migratedConfig.moduleFileExtensions.length && preset) {
        const presetValue = dedupSort(preset.value.moduleFileExtensions || []).join('::');
        const migratedValue = dedupSort(migratedConfig.moduleFileExtensions).join('::');
        if (presetValue === migratedValue) {
            delete migratedConfig.moduleFileExtensions;
        }
    }
    if (migratedConfig.testRegex && preset) {
        migratedConfig.testMatch = null;
    }
    else if (migratedConfig.testMatch && migratedConfig.testMatch.length && preset) {
        const presetValue = dedupSort(preset.value.testMatch || []).join('::');
        const migratedValue = dedupSort(migratedConfig.testMatch).join('::');
        if (presetValue === migratedValue) {
            delete migratedConfig.testMatch;
        }
    }
    if (migratedConfig.transform) {
        Object.keys(migratedConfig.transform).forEach((key) => {
            const val = migratedConfig.transform[key];
            if (typeof val === 'string' && /\/?ts-jest(?:\/preprocessor\.js)?$/.test(val)) {
                ;
                migratedConfig.transform[key] = 'ts-jest';
            }
        });
    }
    if (preset &&
        migratedConfig.transform &&
        stableStringify(migratedConfig.transform) === stableStringify(preset.value.transform)) {
        delete migratedConfig.transform;
    }
    cleanupConfig(actualConfig);
    cleanupConfig(migratedConfig);
    const before = stableStringify(actualConfig);
    const after = stableStringify(migratedConfig);
    if (after === before) {
        process.stderr.write(`
No migration needed for given Jest configuration
    `);
        return;
    }
    const stringify = file.endsWith('.json') ? JSON.stringify : json5_1.stringify;
    const prefix = file.endsWith('.json') ? '"jest": ' : 'module.exports = ';
    if (preset && migratedConfig.transform) {
        footNotes.push(`
I couldn't check if your "transform" value is the same as mine which is: ${stringify(preset.value.transform, undefined, '  ')}
If it is the case, you can safely remove the "transform" from what I've migrated.
`);
    }
    process.stderr.write(`
Migrated Jest configuration:
`);
    process.stdout.write(`${prefix}${stringify(migratedConfig, undefined, '  ')}\n`);
    if (footNotes.length) {
        process.stderr.write(`
${footNotes.join('\n')}
`);
    }
});
exports.run = run;
function cleanupConfig(config) {
    if (config.globals) {
        if (config.globals['ts-jest'] && Object.keys(config.globals['ts-jest']).length === 0) {
            delete config.globals['ts-jest'];
        }
        if (Object.keys(config.globals).length === 0) {
            delete config.globals;
        }
    }
    if (config.transform && Object.keys(config.transform).length === 0) {
        delete config.transform;
    }
    if (config.moduleFileExtensions) {
        config.moduleFileExtensions = dedupSort(config.moduleFileExtensions);
        if (config.moduleFileExtensions.length === 0)
            delete config.moduleFileExtensions;
    }
    if (config.testMatch) {
        config.testMatch = dedupSort(config.testMatch);
        if (config.testMatch.length === 0)
            delete config.testMatch;
    }
    if (config.preset === "ts-jest/presets/default")
        config.preset = presets_1.defaults.name;
}
function dedupSort(arr) {
    return arr
        .filter((s, i, a) => a.findIndex((e) => s.toString() === e.toString()) === i)
        .sort((a, b) => (a.toString() > b.toString() ? 1 : a.toString() < b.toString() ? -1 : 0));
}
const help = () => __awaiter(void 0, void 0, void 0, function* () {
    process.stdout.write(`
Usage:
  ts-jest config:migrate [options] <config-file>

Arguments:
  <config-file>         Can be a js or json Jest config file. If it is a
                        package.json file, the configuration will be read from
                        the "jest" property.

Options:
  --js ts|babel         Process .js files with ts-jest if 'ts' or with
                        babel-jest if 'babel'
  --no-jest-preset      Disable the use of Jest presets
`);
});
exports.help = help;
