"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.processArgv = void 0;
const bs_logger_1 = require("bs-logger");
const yargs_parser_1 = __importDefault(require("yargs-parser"));
const logger_1 = require("../utils/logger");
const VALID_COMMANDS = ['help', 'config:migrate', 'config:init'];
const logger = logger_1.rootLogger.child({ [bs_logger_1.LogContexts.namespace]: 'cli', [bs_logger_1.LogContexts.application]: 'ts-jest' });
function cli(args) {
    return __awaiter(this, void 0, void 0, function* () {
        const parsedArgv = yargs_parser_1.default(args, {
            boolean: ['dry-run', 'jest-preset', 'allow-js', 'diff', 'babel', 'force', 'jsdom'],
            string: ['tsconfig', 'js'],
            count: ['verbose'],
            alias: { verbose: ['v'] },
            default: { jestPreset: true, verbose: 0 },
            coerce: {
                js(val) {
                    const res = val.trim().toLowerCase();
                    if (!['babel', 'ts'].includes(res))
                        throw new Error(`The 'js' option must be 'babel' or 'ts', given: '${val}'.`);
                    return res;
                },
            },
        });
        if (parsedArgv.allowJs != null) {
            if (parsedArgv.js)
                throw new Error("The 'allowJs' and 'js' options cannot be set together.");
            parsedArgv.js = parsedArgv.allowJs ? 'ts' : undefined;
        }
        let command = parsedArgv._.shift();
        const isHelp = command === 'help';
        if (isHelp)
            command = parsedArgv._.shift();
        if (!VALID_COMMANDS.includes(command))
            command = 'help';
        const { run, help } = require(`./${command.replace(/:/g, '/')}`);
        const cmd = isHelp && command !== 'help' ? help : run;
        return cmd(parsedArgv, logger);
    });
}
function processArgv() {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            yield cli(process.argv.slice(2));
            process.exit(0);
        }
        catch (err) {
            logger.fatal(err.message);
            process.exit(1);
        }
    });
}
exports.processArgv = processArgv;
