"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ConfigSet = exports.TS_JEST_OUT_DIR = exports.IGNORE_DIAGNOSTIC_CODES = exports.MY_DIGEST = void 0;
const bs_logger_1 = require("bs-logger");
const fs_1 = require("fs");
const jest_util_1 = require("jest-util");
const json5 = require("json5");
const path_1 = require("path");
const typescript_1 = require("typescript");
const constants_1 = require("../constants");
const hoist_jest_1 = require("../transformers/hoist-jest");
const backports_1 = require("../utils/backports");
const importer_1 = require("../utils/importer");
const json_1 = require("../utils/json");
const logger_1 = require("../utils/logger");
const memoize_1 = require("../utils/memoize");
const messages_1 = require("../utils/messages");
const normalize_slashes_1 = require("../utils/normalize-slashes");
const sha1_1 = require("../utils/sha1");
const ts_error_1 = require("../utils/ts-error");
exports.MY_DIGEST = fs_1.readFileSync(path_1.resolve(__dirname, '..', '..', '.ts-jest-digest'), 'utf8');
exports.IGNORE_DIAGNOSTIC_CODES = [
    6059,
    18002,
    18003,
];
exports.TS_JEST_OUT_DIR = '$$ts-jest$$';
const TARGET_TO_VERSION_MAPPING = {
    [typescript_1.ScriptTarget.ES2018]: 'es2018',
    [typescript_1.ScriptTarget.ES2019]: 'es2019',
    [typescript_1.ScriptTarget.ES2020]: 'es2020',
    [typescript_1.ScriptTarget.ESNext]: 'ESNext',
};
const normalizeRegex = (pattern) => pattern ? (typeof pattern === 'string' ? pattern : pattern.source) : undefined;
const toDiagnosticCode = (code) => { var _a; return code ? (_a = parseInt(`${code}`.trim().replace(/^TS/, ''), 10)) !== null && _a !== void 0 ? _a : undefined : undefined; };
const toDiagnosticCodeList = (items, into = []) => {
    for (let item of items) {
        if (typeof item === 'string') {
            const children = item.trim().split(/\s*,\s*/g);
            if (children.length > 1) {
                toDiagnosticCodeList(children, into);
                continue;
            }
            item = children[0];
        }
        if (!item)
            continue;
        const code = toDiagnosticCode(item);
        if (code && !into.includes(code))
            into.push(code);
    }
    return into;
};
class ConfigSet {
    constructor(jestConfig, parentLogger) {
        var _a, _b, _c, _d;
        this.jestConfig = jestConfig;
        this.parentLogger = parentLogger;
        this.customTransformers = Object.create(null);
        this._overriddenCompilerOptions = {
            sourceMap: true,
            inlineSourceMap: false,
            inlineSources: true,
            declaration: false,
            noEmit: false,
            removeComments: false,
            out: undefined,
            outFile: undefined,
            composite: undefined,
            declarationDir: undefined,
            declarationMap: undefined,
            emitDeclarationOnly: undefined,
            sourceRoot: undefined,
            tsBuildInfoFile: undefined,
        };
        this.logger = this.parentLogger
            ? this.parentLogger.child({ [bs_logger_1.LogContexts.namespace]: 'config' })
            : logger_1.rootLogger.child({ namespace: 'config' });
        this.cwd = path_1.normalize((_a = this.jestConfig.cwd) !== null && _a !== void 0 ? _a : process.cwd());
        this.rootDir = path_1.normalize((_b = this.jestConfig.rootDir) !== null && _b !== void 0 ? _b : this.cwd);
        const tsJestCfg = this.jestConfig.globals && this.jestConfig.globals['ts-jest'];
        const options = tsJestCfg !== null && tsJestCfg !== void 0 ? tsJestCfg : Object.create(null);
        this.compilerModule = importer_1.importer.typescript("Using \"ts-jest\" requires this package to be installed.", (_c = options.compiler) !== null && _c !== void 0 ? _c : 'typescript');
        this.isolatedModules = (_d = options.isolatedModules) !== null && _d !== void 0 ? _d : false;
        this.logger.debug({ compilerModule: this.compilerModule }, 'normalized compiler module config via ts-jest option');
        this._backportJestCfg();
        this._setupTsJestCfg(options);
        this._resolveTsCacheDir();
    }
    _backportJestCfg() {
        const config = backports_1.backportJestConfig(this.logger, this.jestConfig);
        this.logger.debug({ jestConfig: config }, 'normalized jest config');
        this._jestCfg = config;
    }
    _setupTsJestCfg(options) {
        var _a, _b, _c;
        if (!options.babelConfig) {
            this.logger.debug('babel is disabled');
        }
        else {
            const baseBabelCfg = { cwd: this.cwd };
            if (typeof options.babelConfig === 'string') {
                const babelCfgPath = this.resolvePath(options.babelConfig);
                if (path_1.extname(options.babelConfig) === '.js') {
                    this._babelConfig = Object.assign(Object.assign({}, baseBabelCfg), require(babelCfgPath));
                }
                else {
                    this._babelConfig = Object.assign(Object.assign({}, baseBabelCfg), json5.parse(fs_1.readFileSync(babelCfgPath, 'utf-8')));
                }
            }
            else if (typeof options.babelConfig === 'object') {
                this._babelConfig = Object.assign(Object.assign({}, baseBabelCfg), options.babelConfig);
            }
            else {
                this._babelConfig = baseBabelCfg;
            }
            this.logger.debug({ babelConfig: this._babelConfig }, 'normalized babel config via ts-jest option');
        }
        if (!this._babelConfig) {
            this._overriddenCompilerOptions.module = this.compilerModule.ModuleKind.CommonJS;
        }
        else {
            this._babelJestTransformers = importer_1.importer
                .babelJest("Using \"babel-jest\" requires this package to be installed.")
                .createTransformer(this._babelConfig);
            this.logger.debug('created babel-jest transformer');
        }
        const diagnosticsOpt = (_a = options.diagnostics) !== null && _a !== void 0 ? _a : true;
        const ignoreList = [...exports.IGNORE_DIAGNOSTIC_CODES];
        if (typeof diagnosticsOpt === 'object') {
            const { ignoreCodes } = diagnosticsOpt;
            if (ignoreCodes) {
                Array.isArray(ignoreCodes) ? ignoreList.push(...ignoreCodes) : ignoreList.push(ignoreCodes);
            }
            this._diagnostics = {
                pretty: (_b = diagnosticsOpt.pretty) !== null && _b !== void 0 ? _b : true,
                ignoreCodes: toDiagnosticCodeList(ignoreList),
                pathRegex: normalizeRegex(diagnosticsOpt.pathRegex),
                throws: !diagnosticsOpt.warnOnly,
            };
        }
        else {
            this._diagnostics = {
                ignoreCodes: diagnosticsOpt ? toDiagnosticCodeList(ignoreList) : [],
                pretty: true,
                throws: diagnosticsOpt,
            };
        }
        this.logger.debug({ diagnostics: this._diagnostics }, 'normalized diagnostics config via ts-jest option');
        const tsconfigOpt = options.tsconfig;
        const configFilePath = typeof tsconfigOpt === 'string' ? this.resolvePath(tsconfigOpt) : undefined;
        this.parsedTsConfig = this._resolveTsConfig(typeof tsconfigOpt === 'object' ? tsconfigOpt : undefined, configFilePath);
        this.raiseDiagnostics(this.parsedTsConfig.errors, configFilePath);
        this.logger.debug({ tsconfig: this.parsedTsConfig }, 'normalized typescript config via ts-jest option');
        const { astTransformers } = options;
        this.customTransformers = {
            before: [hoist_jest_1.factory(this)],
        };
        if (astTransformers) {
            const resolveTransformers = (transformers) => transformers.map((transformer) => {
                let transformerPath;
                if (typeof transformer === 'string') {
                    transformerPath = this.resolvePath(transformer, { nodeResolve: true });
                    return require(transformerPath).factory(this);
                }
                else {
                    transformerPath = this.resolvePath(transformer.path, { nodeResolve: true });
                    return require(transformerPath).factory(this, transformer.options);
                }
            });
            if (astTransformers.before) {
                (_c = this.customTransformers.before) === null || _c === void 0 ? void 0 : _c.push(...resolveTransformers(astTransformers.before));
            }
            if (astTransformers.after) {
                this.customTransformers = Object.assign(Object.assign({}, this.customTransformers), { after: resolveTransformers(astTransformers.after) });
            }
            if (astTransformers.afterDeclarations) {
                this.customTransformers = Object.assign(Object.assign({}, this.customTransformers), { afterDeclarations: resolveTransformers(astTransformers.afterDeclarations) });
            }
        }
        this.logger.debug({ customTransformers: this.customTransformers }, 'normalized custom AST transformers via ts-jest option');
        if (options.stringifyContentPathRegex) {
            this._stringifyContentRegExp =
                typeof options.stringifyContentPathRegex === 'string'
                    ? new RegExp(normalizeRegex(options.stringifyContentPathRegex))
                    : options.stringifyContentPathRegex;
            this.logger.debug({ stringifyContentPathRegex: this._stringifyContentRegExp }, 'normalized stringifyContentPathRegex config via ts-jest option');
        }
    }
    _resolveTsCacheDir() {
        if (!this._jestCfg.cache) {
            this.logger.debug('file caching disabled');
            return undefined;
        }
        const cacheSuffix = sha1_1.sha1(json_1.stringify({
            version: this.compilerModule.version,
            digest: this.tsJestDigest,
            compilerModule: this.compilerModule,
            compilerOptions: this.parsedTsConfig.options,
            isolatedModules: this.isolatedModules,
            diagnostics: this._diagnostics,
        }));
        const res = path_1.join(this._jestCfg.cacheDirectory, 'ts-jest', cacheSuffix.substr(0, 2), cacheSuffix.substr(2));
        this.logger.debug({ cacheDirectory: res }, 'will use file caching');
        this.tsCacheDir = res;
    }
    _resolveTsConfig(compilerOptions, resolvedConfigFile) {
        var _a;
        let config = { compilerOptions: Object.create(null) };
        let basePath = normalize_slashes_1.normalizeSlashes(this.rootDir);
        const ts = this.compilerModule;
        const configFileName = resolvedConfigFile
            ? normalize_slashes_1.normalizeSlashes(resolvedConfigFile)
            : ts.findConfigFile(normalize_slashes_1.normalizeSlashes(this.rootDir), ts.sys.fileExists);
        if (configFileName) {
            this.logger.debug({ tsConfigFileName: configFileName }, 'readTsConfig(): reading', configFileName);
            const result = ts.readConfigFile(configFileName, ts.sys.readFile);
            if (result.error) {
                return { errors: [result.error], fileNames: [], options: {} };
            }
            config = result.config;
            basePath = normalize_slashes_1.normalizeSlashes(path_1.dirname(configFileName));
        }
        config.compilerOptions = Object.assign(Object.assign({}, config.compilerOptions), compilerOptions);
        const result = ts.parseJsonConfigFileContent(config, ts.sys, basePath, undefined, configFileName);
        const { _overriddenCompilerOptions: forcedOptions } = this;
        const finalOptions = result.options;
        if (finalOptions.target === undefined) {
            finalOptions.target = ts.ScriptTarget.ES2015;
        }
        const target = finalOptions.target;
        const defaultModule = [ts.ScriptTarget.ES3, ts.ScriptTarget.ES5].includes(target)
            ? ts.ModuleKind.CommonJS
            : ts.ModuleKind.ESNext;
        const moduleValue = finalOptions.module == null ? defaultModule : finalOptions.module;
        if ('module' in forcedOptions &&
            moduleValue !== forcedOptions.module &&
            !(finalOptions.esModuleInterop || finalOptions.allowSyntheticDefaultImports)) {
            result.errors.push({
                code: 151001,
                messageText: "If you have issues related to imports, you should consider setting `esModuleInterop` to `true` in your TypeScript configuration file (usually `tsconfig.json`). See https://blogs.msdn.microsoft.com/typescript/2018/01/31/announcing-typescript-2-7/#easier-ecmascript-module-interoperability for more information.",
                category: ts.DiagnosticCategory.Message,
                file: undefined,
                start: undefined,
                length: undefined,
            });
            if (!('allowSyntheticDefaultImports' in config.compilerOptions)) {
                finalOptions.allowSyntheticDefaultImports = true;
            }
        }
        if (finalOptions.allowJs && !finalOptions.outDir) {
            finalOptions.outDir = exports.TS_JEST_OUT_DIR;
        }
        for (const key of Object.keys(forcedOptions)) {
            const val = forcedOptions[key];
            if (val === undefined) {
                delete finalOptions[key];
            }
            else {
                finalOptions[key] = val;
            }
        }
        const nodeJsVer = process.version;
        const compilationTarget = result.options.target;
        if (!this._babelConfig &&
            ((nodeJsVer.startsWith('v10') && compilationTarget > typescript_1.ScriptTarget.ES2018) ||
                (nodeJsVer.startsWith('v12') && compilationTarget > typescript_1.ScriptTarget.ES2019))) {
            const message = messages_1.interpolate("There is a mismatch between your NodeJs version {{nodeJsVer}} and your TypeScript target {{compilationTarget}}. This might lead to some unexpected errors when running tests with `ts-jest`. To fix this, you can check https://github.com/microsoft/TypeScript/wiki/Node-Target-Mapping", {
                nodeJsVer: process.version,
                compilationTarget: (_a = config.compilerOptions.target) !== null && _a !== void 0 ? _a : TARGET_TO_VERSION_MAPPING[compilationTarget],
            });
            this.logger.warn(message);
        }
        return result;
    }
    get babelConfig() {
        return this._babelConfig;
    }
    get babelJestTransformer() {
        return this._babelJestTransformers;
    }
    get tsJestDigest() {
        return exports.MY_DIGEST;
    }
    get hooks() {
        let hooksFile = process.env.TS_JEST_HOOKS;
        if (hooksFile) {
            hooksFile = path_1.resolve(this.cwd, hooksFile);
            return importer_1.importer.tryTheseOr(hooksFile, {});
        }
        return {};
    }
    get isTestFile() {
        const matchablePatterns = [...this._jestCfg.testMatch, ...this._jestCfg.testRegex].filter((pattern) => pattern instanceof RegExp || typeof pattern === 'string');
        if (!matchablePatterns.length) {
            matchablePatterns.push(...constants_1.DEFAULT_JEST_TEST_MATCH);
        }
        const stringPatterns = matchablePatterns.filter((pattern) => typeof pattern === 'string');
        const isMatch = jest_util_1.globsToMatcher(stringPatterns);
        return (fileName) => matchablePatterns.some((pattern) => (typeof pattern === 'string' ? isMatch(fileName) : pattern.test(fileName)));
    }
    shouldStringifyContent(filePath) {
        return this._stringifyContentRegExp ? this._stringifyContentRegExp.test(filePath) : false;
    }
    raiseDiagnostics(diagnostics, filePath, logger) {
        const { ignoreCodes } = this._diagnostics;
        const { DiagnosticCategory } = this.compilerModule;
        const filteredDiagnostics = filePath && !this.shouldReportDiagnostics(filePath)
            ? []
            : diagnostics.filter((diagnostic) => {
                var _a;
                if (((_a = diagnostic.file) === null || _a === void 0 ? void 0 : _a.fileName) && !this.shouldReportDiagnostics(diagnostic.file.fileName)) {
                    return false;
                }
                return !ignoreCodes.includes(diagnostic.code);
            });
        if (!filteredDiagnostics.length)
            return;
        const error = this._createTsError(filteredDiagnostics);
        const importantCategories = [DiagnosticCategory.Warning, DiagnosticCategory.Error];
        if (this._diagnostics.throws && filteredDiagnostics.some((d) => importantCategories.includes(d.category))) {
            throw error;
        }
        logger ? logger.warn({ error }, error.message) : this.logger.warn({ error }, error.message);
    }
    shouldReportDiagnostics(filePath) {
        const { pathRegex } = this._diagnostics;
        if (pathRegex) {
            const regex = new RegExp(pathRegex);
            return regex.test(filePath);
        }
        else {
            return true;
        }
    }
    _createTsError(diagnostics) {
        const formatDiagnostics = this._diagnostics.pretty
            ? this.compilerModule.formatDiagnosticsWithColorAndContext
            : this.compilerModule.formatDiagnostics;
        const diagnosticHost = {
            getNewLine: () => '\n',
            getCurrentDirectory: () => this.cwd,
            getCanonicalFileName: (path) => path,
        };
        const diagnosticText = formatDiagnostics(diagnostics, diagnosticHost);
        const diagnosticCodes = diagnostics.map((x) => x.code);
        return new ts_error_1.TSError(diagnosticText, diagnosticCodes);
    }
    resolvePath(inputPath, { throwIfMissing = true, nodeResolve = false } = {}) {
        let path = inputPath;
        let nodeResolved = false;
        if (path.startsWith('<rootDir>')) {
            path = path_1.resolve(path_1.join(this.rootDir, path.substr(9)));
        }
        else if (!path_1.isAbsolute(path)) {
            if (!path.startsWith('.') && nodeResolve) {
                try {
                    path = require.resolve(path);
                    nodeResolved = true;
                }
                catch (_) { }
            }
            if (!nodeResolved) {
                path = path_1.resolve(this.cwd, path);
            }
        }
        if (!nodeResolved && nodeResolve) {
            try {
                path = require.resolve(path);
                nodeResolved = true;
            }
            catch (_) { }
        }
        if (throwIfMissing && !fs_1.existsSync(path)) {
            throw new Error(messages_1.interpolate("File not found: {{inputPath}} (resolved as: {{resolvedPath}})", { inputPath, resolvedPath: path }));
        }
        this.logger.debug({ fromPath: inputPath, toPath: path }, 'resolved path from', inputPath, 'to', path);
        return path;
    }
}
__decorate([
    memoize_1.Memoize()
], ConfigSet.prototype, "tsJestDigest", null);
__decorate([
    memoize_1.Memoize()
], ConfigSet.prototype, "hooks", null);
__decorate([
    memoize_1.Memoize()
], ConfigSet.prototype, "isTestFile", null);
exports.ConfigSet = ConfigSet;
