"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.pathsToModuleNameMapper = void 0;
const bs_logger_1 = require("bs-logger");
const logger_1 = require("../utils/logger");
const messages_1 = require("../utils/messages");
const escapeRegex = (str) => str.replace(/[-\\^$*+?.()|[\]{}]/g, '\\$&');
const logger = logger_1.rootLogger.child({ [bs_logger_1.LogContexts.namespace]: 'path-mapper' });
const pathsToModuleNameMapper = (mapping, { prefix = '' } = Object.create(null)) => {
    const jestMap = {};
    for (const fromPath of Object.keys(mapping)) {
        let pattern;
        const toPaths = mapping[fromPath];
        if (toPaths.length === 0) {
            logger.warn(messages_1.interpolate("Not mapping \"{{path}}\" because it has no target.", { path: fromPath }));
            continue;
        }
        const segments = fromPath.split(/\*/g);
        if (segments.length === 1) {
            const paths = toPaths.map((target) => {
                const enrichedPrefix = prefix !== '' && !prefix.endsWith('/') ? `${prefix}/` : prefix;
                return `${enrichedPrefix}${target}`;
            });
            pattern = `^${escapeRegex(fromPath)}$`;
            jestMap[pattern] = paths.length === 1 ? paths[0] : paths;
        }
        else if (segments.length === 2) {
            const paths = toPaths.map((target) => {
                const enrichedPrefix = prefix !== '' && !prefix.endsWith('/') ? `${prefix}/` : prefix;
                return `${enrichedPrefix}${target.replace(/\*/g, '$1')}`;
            });
            pattern = `^${escapeRegex(segments[0])}(.*)${escapeRegex(segments[1])}$`;
            jestMap[pattern] = paths.length === 1 ? paths[0] : paths;
        }
        else {
            logger.warn(messages_1.interpolate("Not mapping \"{{path}}\" because it has more than one star (`*`).", { path: fromPath }));
        }
    }
    return jestMap;
};
exports.pathsToModuleNameMapper = pathsToModuleNameMapper;
