"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.factory = exports.version = exports.name = void 0;
const bs_logger_1 = require("bs-logger");
const HOIST_METHODS = ['mock', 'unmock', 'enableAutomock', 'disableAutomock', 'deepUnmock'];
const JEST_GLOBALS_MODULE_NAME = '@jest/globals';
const JEST_GLOBAL_NAME = 'jest';
const ROOT_LEVEL_AST = 1;
exports.name = 'hoisting-jest-mock';
exports.version = 4;
function factory(cs) {
    const logger = cs.logger.child({ namespace: 'ts-hoisting' });
    const ts = cs.compilerModule;
    const importNames = [];
    function shouldHoistExpression(node) {
        if (ts.isCallExpression(node) &&
            ts.isPropertyAccessExpression(node.expression) &&
            HOIST_METHODS.includes(node.expression.name.text)) {
            if (importNames.length) {
                return ((ts.isIdentifier(node.expression.expression) && importNames.includes(node.expression.expression.text)) ||
                    (ts.isPropertyAccessExpression(node.expression.expression) &&
                        ts.isIdentifier(node.expression.expression.expression) &&
                        importNames.includes(node.expression.expression.expression.text)) ||
                    shouldHoistExpression(node.expression.expression));
            }
            else {
                return ((ts.isIdentifier(node.expression.expression) && node.expression.expression.text === JEST_GLOBAL_NAME) ||
                    shouldHoistExpression(node.expression.expression));
            }
        }
        return false;
    }
    function shouldHoistNode(node) {
        return ts.isExpressionStatement(node) && shouldHoistExpression(node.expression);
    }
    function isJestGlobalImport(node) {
        return (ts.isImportDeclaration(node) &&
            ts.isStringLiteral(node.moduleSpecifier) &&
            node.moduleSpecifier.text === JEST_GLOBALS_MODULE_NAME);
    }
    function createVisitor(ctx, _) {
        let level = 0;
        const hoisted = [];
        const enter = () => {
            level++;
            if (hoisted[level]) {
                hoisted[level].splice(0, hoisted[level].length);
            }
        };
        const exit = () => level--;
        const hoist = (node) => {
            if (hoisted[level]) {
                hoisted[level].push(node);
            }
            else {
                hoisted[level] = [node];
            }
        };
        const visitor = (node) => {
            var _a, _b;
            enter();
            let resultNode = ts.visitEachChild(node, visitor, ctx);
            if (isJestGlobalImport(resultNode) && ((_a = resultNode.importClause) === null || _a === void 0 ? void 0 : _a.namedBindings) &&
                (ts.isNamespaceImport(resultNode.importClause.namedBindings) ||
                    ts.isNamedImports(resultNode.importClause.namedBindings))) {
                const { namedBindings } = resultNode.importClause;
                const jestImportName = ts.isNamespaceImport(namedBindings)
                    ? namedBindings.name.text
                    : (_b = namedBindings.elements.find((element) => { var _a; return element.name.text === JEST_GLOBAL_NAME || ((_a = element.propertyName) === null || _a === void 0 ? void 0 : _a.text) === JEST_GLOBAL_NAME; })) === null || _b === void 0 ? void 0 : _b.name.text;
                if (jestImportName) {
                    importNames.push(jestImportName);
                }
            }
            if (hoisted[level] && hoisted[level].length) {
                const hoistedStmts = hoisted[level];
                const otherStmts = resultNode.statements.filter((s) => !hoistedStmts.includes(s) && !isJestGlobalImport(s));
                const newNode = ts.getMutableClone(resultNode);
                const newStatements = [...hoistedStmts, ...otherStmts];
                if (level === ROOT_LEVEL_AST) {
                    const jestGlobalsImportStmts = resultNode.statements.filter((s) => isJestGlobalImport(s));
                    resultNode = Object.assign(Object.assign({}, newNode), { statements: ts.createNodeArray([...jestGlobalsImportStmts, ...newStatements]) });
                }
                else {
                    resultNode = Object.assign(Object.assign({}, newNode), { statements: ts.createNodeArray(newStatements) });
                }
            }
            exit();
            if (shouldHoistNode(resultNode)) {
                hoist(resultNode);
            }
            return resultNode;
        };
        return visitor;
    }
    return (ctx) => logger.wrap({ [bs_logger_1.LogContexts.logLevel]: bs_logger_1.LogLevels.debug, call: null }, 'visitSourceFileNode(): hoisting', (sf) => ts.visitNode(sf, createVisitor(ctx, sf)));
}
exports.factory = factory;
