"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.factory = exports.version = exports.name = void 0;
const bs_logger_1 = require("bs-logger");
const path_1 = require("path");
exports.name = 'path-mapping';
exports.version = 1;
const isBaseDir = (base, dir) => { var _a; return !((_a = path_1.relative(base, dir)) === null || _a === void 0 ? void 0 : _a.startsWith('.')); };
function factory(cs) {
    var _a;
    const logger = cs.logger.child({ namespace: 'ts-path-mapping' });
    const ts = cs.compilerModule;
    const compilerOptions = cs.parsedTsConfig.options;
    const rootDirs = (_a = compilerOptions.rootDirs) === null || _a === void 0 ? void 0 : _a.filter(path_1.isAbsolute);
    const isDynamicImport = (node) => ts.isCallExpression(node) && node.expression.kind === ts.SyntaxKind.ImportKeyword;
    const isRequire = (node) => ts.isCallExpression(node) &&
        ts.isIdentifier(node.expression) &&
        node.expression.text === 'require' &&
        ts.isStringLiteral(node.arguments[0]) &&
        node.arguments.length === 1;
    const createVisitor = (ctx, sf) => {
        const { fileName } = sf;
        const fileDir = path_1.normalize(path_1.dirname(fileName));
        const rewritePath = (importPath) => {
            let p = importPath;
            const { resolvedModule } = ts.resolveModuleName(importPath, fileName, compilerOptions, ts.sys);
            if (resolvedModule) {
                const { resolvedFileName } = resolvedModule;
                let filePath = fileDir;
                let modulePath = path_1.dirname(resolvedFileName);
                if (rootDirs) {
                    let fileRootDir = '';
                    let moduleRootDir = '';
                    for (const rootDir of rootDirs) {
                        if (isBaseDir(rootDir, resolvedFileName) && rootDir.length > moduleRootDir.length)
                            moduleRootDir = rootDir;
                        if (isBaseDir(rootDir, fileName) && rootDir.length > fileRootDir.length)
                            fileRootDir = rootDir;
                    }
                    if (fileRootDir && moduleRootDir) {
                        filePath = path_1.relative(fileRootDir, filePath);
                        modulePath = path_1.relative(moduleRootDir, modulePath);
                    }
                }
                p = path_1.normalize(path_1.join(path_1.relative(filePath, modulePath), path_1.basename(resolvedFileName)));
                p = p.startsWith('.') ? p : `./${p}`;
            }
            return p;
        };
        const visitor = (node) => {
            let rewrittenPath;
            const newNode = ts.getMutableClone(node);
            if (isDynamicImport(node) || isRequire(node)) {
                rewrittenPath = rewritePath(node.arguments[0].text);
                return Object.assign(Object.assign({}, newNode), { arguments: ts.createNodeArray([ts.createStringLiteral(rewrittenPath)]) });
            }
            if (ts.isExternalModuleReference(node) && ts.isStringLiteral(node.expression)) {
                rewrittenPath = rewritePath(node.expression.text);
                return ts.updateExternalModuleReference(newNode, ts.createLiteral(rewrittenPath));
            }
            if (ts.isImportDeclaration(node) && ts.isStringLiteral(node.moduleSpecifier)) {
                rewrittenPath = rewritePath(node.moduleSpecifier.text);
                return Object.assign(Object.assign({}, newNode), { moduleSpecifier: ts.createLiteral(rewrittenPath) });
            }
            if (ts.isExportDeclaration(node) && node.moduleSpecifier && ts.isStringLiteral(node.moduleSpecifier)) {
                rewrittenPath = rewritePath(node.moduleSpecifier.text);
                return Object.assign(Object.assign({}, newNode), { moduleSpecifier: ts.createLiteral(rewrittenPath) });
            }
            if (ts.isImportTypeNode(node) &&
                ts.isLiteralTypeNode(node.argument) &&
                ts.isStringLiteral(node.argument.literal)) {
                rewrittenPath = rewritePath(node.argument.literal.text);
                return Object.assign(Object.assign({}, newNode), { argument: ts.createLiteralTypeNode(ts.createStringLiteral(rewrittenPath)) });
            }
            return ts.visitEachChild(node, visitor, ctx);
        };
        return visitor;
    };
    return (ctx) => logger.wrap({ [bs_logger_1.LogContexts.logLevel]: bs_logger_1.LogLevels.debug, call: null }, 'visitSourceFileNode(): path mapping', (sf) => ts.visitNode(sf, createVisitor(ctx, sf)));
}
exports.factory = factory;
