"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.__requireModule = exports.importer = exports.Importer = void 0;
const logger_1 = require("./logger");
const memoize_1 = require("./memoize");
const messages_1 = require("./messages");
const version_checkers_1 = require("./version-checkers");
const logger = logger_1.rootLogger.child({ namespace: 'Importer' });
const passThru = (action) => (input) => {
    action();
    return input;
};
class Importer {
    constructor(_patches = {}) {
        this._patches = _patches;
    }
    static get instance() {
        logger.debug('creating Importer singleton');
        return new Importer({
            '@babel/core': [passThru(version_checkers_1.VersionCheckers.babelCore.warn)],
            'babel-jest': [passThru(version_checkers_1.VersionCheckers.babelJest.warn)],
            typescript: [passThru(version_checkers_1.VersionCheckers.typescript.warn)],
            jest: [passThru(version_checkers_1.VersionCheckers.jest.warn)],
        });
    }
    babelJest(why) {
        return this._import(why, 'babel-jest');
    }
    babelCore(why) {
        return this._import(why, '@babel/core');
    }
    typescript(why, which) {
        return this._import(why, which);
    }
    tryThese(moduleName, ...fallbacks) {
        let name;
        let loaded;
        const tries = [moduleName, ...fallbacks];
        while ((name = tries.shift()) !== undefined) {
            const req = requireWrapper(name);
            const contextReq = Object.assign({}, req);
            delete contextReq.exports;
            if (req.exists) {
                loaded = req;
                if (loaded.error) {
                    logger.error({ requireResult: contextReq }, `failed loading module '${name}'`, loaded.error.message);
                }
                else {
                    logger.debug({ requireResult: contextReq }, 'loaded module', name);
                    loaded.exports = this._patch(name, loaded.exports);
                }
                break;
            }
            else {
                logger.debug({ requireResult: contextReq }, `module '${name}' not found`);
                continue;
            }
        }
        return loaded;
    }
    tryTheseOr(moduleNames, missingResult, allowLoadError = false) {
        const args = Array.isArray(moduleNames) ? moduleNames : [moduleNames];
        const result = this.tryThese(...args);
        if (!result)
            return missingResult;
        if (!result.error)
            return result.exports;
        if (allowLoadError)
            return missingResult;
        throw result.error;
    }
    _patch(name, unpatched) {
        if (name in this._patches) {
            logger.debug('patching', name);
            return this._patches[name].reduce((mod, patcher) => patcher(mod), unpatched);
        }
        return unpatched;
    }
    _import(why, moduleName, { alternatives = [], installTip = moduleName } = {}) {
        const res = this.tryThese(moduleName, ...alternatives);
        if (res && res.exists) {
            if (!res.error)
                return res.exports;
            throw new Error(messages_1.interpolate("Loading module {{module}} failed with error: {{error}}", { module: res.given, error: res.error.message }));
        }
        const msg = alternatives.length ? "Unable to load any of these modules: {{module}}. {{reason}}. To fix it:\n{{fix}}" : "Unable to load the module {{module}}. {{reason}} To fix it:\n{{fix}}";
        const loadModule = [moduleName, ...alternatives].map((m) => `"${m}"`).join(', ');
        if (typeof installTip === 'string') {
            installTip = [{ module: installTip, label: `install "${installTip}"` }];
        }
        const fix = installTip
            .map((tip) => `    ${installTip.length === 1 ? '↳' : '•'} ${messages_1.interpolate("{{label}}: `npm i -D {{module}}` (or `yarn add --dev {{module}}`)", tip)}`)
            .join('\n');
        throw new Error(messages_1.interpolate(msg, {
            module: loadModule,
            reason: why,
            fix,
        }));
    }
}
__decorate([
    memoize_1.Memoize((...args) => args.join(':'))
], Importer.prototype, "tryThese", null);
__decorate([
    memoize_1.Memoize((name) => name)
], Importer.prototype, "_patch", null);
__decorate([
    memoize_1.Memoize()
], Importer, "instance", null);
exports.Importer = Importer;
exports.importer = Importer.instance;
function requireWrapper(moduleName) {
    let path;
    let exists = false;
    try {
        path = resolveModule(moduleName);
        exists = true;
    }
    catch (error) {
        return { error, exists, given: moduleName };
    }
    const result = { exists, path, given: moduleName };
    try {
        result.exports = requireModule(path);
    }
    catch (error) {
        try {
            result.exports = requireModule(moduleName);
        }
        catch (error) {
            result.error = error;
        }
    }
    return result;
}
let requireModule = (mod) => require(mod);
let resolveModule = (mod) => require.resolve(mod, { paths: [process.cwd(), __dirname] });
function __requireModule(localRequire, localResolve) {
    requireModule = localRequire;
    resolveModule = localResolve;
}
exports.__requireModule = __requireModule;
