"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Memoize = void 0;
const cacheProp = Symbol.for('[memoize]');
function Memoize(keyBuilder) {
    return (_, propertyKey, descriptor) => {
        if (descriptor.value != null) {
            descriptor.value = memoize(propertyKey, descriptor.value, keyBuilder || ((v) => v));
        }
        else if (descriptor.get != null) {
            descriptor.get = memoize(propertyKey, descriptor.get, keyBuilder || (() => propertyKey));
        }
    };
}
exports.Memoize = Memoize;
function ensureCache(target, reset = false) {
    if (reset || !target[cacheProp]) {
        Object.defineProperty(target, cacheProp, {
            value: Object.create(null),
            configurable: true,
        });
    }
    return target[cacheProp];
}
function ensureChildCache(target, key, reset = false) {
    const dict = ensureCache(target);
    if (reset || !dict[key]) {
        dict[key] = new Map();
    }
    return dict[key];
}
function memoize(namespace, func, keyBuilder) {
    return function (...args) {
        const cache = ensureChildCache(this, namespace);
        const key = keyBuilder.apply(this, args);
        if (cache.has(key))
            return cache.get(key);
        const res = func.apply(this, args);
        cache.set(key, res);
        return res;
    };
}
